ALERTS = "alerts"
CHILD_STOPS = "child_stops"
COUNT = "count"
DIRECTION = "direction"
DIRECTIONS = "directions"
ERROR = "error"
FEEDS = "feeds"
GROUP_ID = "group_id"
HREF = "href"
LAST_STOP = "last_stop"
PARENT_STOP = "parent_stop"
PERIODICITY = "periodicity"
ROUTE = "route"
ROUTES = "routes"
SERVICE_MAPS = "service_maps"
STATUS = "status"
STOP = "stop"
STOP_ID = "stop_id"
STOP_TIMES = "stop_times"
STOPS = "stops"
TRIP = "trip"
UPDATES = "updates"
