__version__ = "0.4.2"

# The following are set by Travis CI at build time.
__build_number__ = None
__build_timestamp__ = None
__build_href__ = None
__git_commit_hash__ = None

# NOTE: the following were set by Travis CI
__version__ = "0.4.2"
__build_number__ = 738
__build_timestamp__ = 1587235180
__build_href__ = "https://travis-ci.org/jamespfennell/transiter/jobs/676666848"
__git_commit_hash__ = "ae629327d29759f0b4554cc2b4072ed52dfa9575"
